#include "globals.h"

#define MAX_ERRORS      15
#define MAX_MESSAGES    15

static Cursor watchCursor;
static Widget errorDialog, messageDialog;
static XmString errorMsg, messageMsg;
static int nErrors, nMessages;
extern struct FileManager* fmgr;


void createErrorAndWarningDialogs( w )
  Widget w;
{
  nErrors = 0;
  nMessages = 0;
  errorMsg = XmStringCreateLtoR( "", XmSTRING_DEFAULT_CHARSET );
  messageMsg = XmStringCreateLtoR( "", XmSTRING_DEFAULT_CHARSET );

  errorDialog = XmCreateErrorDialog( w, "errorDialog", NULL, 0 );
  XtVaSetValues( XtParent(errorDialog), XtNtitle, "*** ERROR ***", NULL );
  XtUnmanageChild( XmMessageBoxGetChild(errorDialog,XmDIALOG_CANCEL_BUTTON) );
  XtUnmanageChild( XmMessageBoxGetChild(errorDialog,XmDIALOG_HELP_BUTTON) );

  messageDialog = XmCreateMessageDialog( w, "messageDialog", NULL, 0 );
  XtVaSetValues( XtParent(messageDialog), XtNtitle, "*** MESSAGE ***", NULL );
  XtUnmanageChild( XmMessageBoxGetChild(messageDialog,XmDIALOG_CANCEL_BUTTON) );
  XtUnmanageChild( XmMessageBoxGetChild(messageDialog,XmDIALOG_HELP_BUTTON) );

  watchCursor = XCreateFontCursor( XtDisplay(w), XC_watch );
}

void traverseCB( w, x1, x2 )
  Widget w;
  XtPointer x1, x2;
{
  XmProcessTraversal( w, XmTRAVERSE_NEXT_TAB_GROUP );
}

int issuePipeCommand( cmd )
  char *cmd;
{
  FILE *fid;
  char buffer[1056], buff[1056];
  int n, nTotal = 0;

#ifdef DEBUG
  printf( "COMMAND = %s\n", cmd );
#endif

  if( (fid = popen( cmd, "r" )) == (FILE *)NULL )
  {
    popupErrorDialog( "popen() failed..." );
    return -1;
  }

  memset( (void *)buffer, '\0', sizeof(buffer) );
  memset( (void *)buff, '\0', sizeof(buff) );
  strcpy( buff, "" );
  while( (n = read( fileno(fid), buffer, sizeof(buffer)-1 )) > 0 )
  {
    nTotal += n;
    strncat( buff, buffer, n );
    memset( (void *)buffer, '\0', sizeof(buffer) );
  }

  if( nTotal > 0 )
  {
    buff[nTotal] = '\0';
    popupErrorDialog( buff );
  }

  pclose( fid );
  return nTotal;
}

void setWatchCursor( )
{
  int i, num = numFM(fmgr);
  for( i = 0; i < num; i++ )
  {
    Widget w = getIthFM(fmgr,i)->parent;
    if( XtIsRealized(w) )
    {
      XDefineCursor( XtDisplay(w), XtWindow(w), watchCursor );
      XFlush( XtDisplay(w) );
    }
  }
}

void unsetWatchCursor( )
{
  int i, num = numFM(fmgr);
  for( i = 0; i < num; i++ )
  {
    Widget w = getIthFM(fmgr,i)->parent;
    if( XtIsRealized(w) )
    {
      XUndefineCursor( XtDisplay(w), XtWindow(w) );
      XFlush( XtDisplay(w) );
    }
  }
}

void setWatchCursorForFM( f )
  struct FileManager* f;
{
  int i, num = numFM(fmgr);
  for( i = 0; i < num; i++ )
  {
    Widget w = getIthFM(fmgr,i)->parent;
    if( w == f->parent && XtIsRealized(w) )
    {
      XDefineCursor( XtDisplay(w), XtWindow(w), watchCursor );
      XFlush( XtDisplay(w) );
    }
  }
}

void unsetWatchCursorForFM( f )
  struct FileManager* f;
{
  int i, num = numFM(fmgr);
  for( i = 0; i < num; i++ )
  {
    Widget w = getIthFM(fmgr,i)->parent;
    if( w == f->parent && XtIsRealized(w) )
    {
      XUndefineCursor( XtDisplay(w), XtWindow(w) );
      XFlush( XtDisplay(w) );
    }
  }
}

void popupErrorDialog( buffer )
  char *buffer;
{
  XmString xms;
  strcat( buffer, "\n" );
  xms = XmStringCreateLtoR( buffer, XmSTRING_DEFAULT_CHARSET );

  if( XtIsManaged(errorDialog) )
  {
        if( nErrors > MAX_ERRORS )
        {
          fprintf( stderr, "*** Number of errors exceeds limit:  max = %d\n", MAX_ERRORS );
          fprintf( stderr, "*** Message = %s\n", buffer );
        }
        else
        {
          nErrors++;
          errorMsg = XmStringConcat( errorMsg, xms );
          XtVaSetValues( errorDialog, XmNmessageString, errorMsg, NULL );
        }
        XRaiseWindow( XtDisplay(errorDialog), XtWindow(errorDialog) );
  }
  else
  {
        nErrors = 1;
        XmStringFree( errorMsg );
        errorMsg = XmStringCopy( xms );
        XtVaSetValues( errorDialog, XmNmessageString, xms, NULL );
        XtManageChild( errorDialog );
        XtPopup( XtParent(errorDialog), XtGrabExclusive );
        XRaiseWindow( XtDisplay(errorDialog), XtWindow(errorDialog) );
  }

  XmStringFree( xms );
}


void popupMessageDialog( buffer )
  char *buffer;
{
  XmString xms;
  strcat( buffer, "\n" );
  xms = XmStringCreateLtoR( buffer, XmSTRING_DEFAULT_CHARSET );

  if( XtIsManaged(messageDialog) )
  {
        if( nMessages > MAX_MESSAGES )
        {
          fprintf( stderr, "*** Number of messages exceeds limit:  max = %d\n", MAX_MESSAGES );
          fprintf( stderr, "*** Message = %s\n", buffer );
        }
        else
        {
          nMessages++;
          messageMsg = XmStringConcat( messageMsg, xms );
          XtVaSetValues( messageDialog, XmNmessageString, messageMsg, NULL );
        }
        XRaiseWindow( XtDisplay(messageDialog), XtWindow(messageDialog) );
  }
  else
  {
        nMessages = 1;
        XmStringFree( messageMsg );
        messageMsg = XmStringCopy( xms );
        XtVaSetValues( messageDialog, XmNmessageString, xms, NULL );
        XtManageChild( messageDialog );
        XtPopup( XtParent(messageDialog), XtGrabNonexclusive );
        XRaiseWindow( XtDisplay(messageDialog), XtWindow(messageDialog) );
  }

  XmStringFree( xms );
}

char* createPausemeFile()
{
  FILE *fd;
  static char tmpFilename[25];

  if( strcmp( tmpFilename, "" ) )
  {
    if( access( tmpFilename, F_OK ) == 0 )
    {
      if( access( tmpFilename, X_OK ) == 0 )
	return tmpFilename;
      else
        chmod( tmpFilename, 0750 );
    }
  }

  sprintf( tmpFilename, "/tmp/pauseme.%d", getpid() );
  if( (fd = fopen( tmpFilename, "w" )) == NULL )
  {
    char errorMsg[50];
    sprintf( errorMsg, "Cannot open %s...", tmpFilename );
    popupErrorDialog( errorMsg );
    return (char *)NULL;
  }

  fprintf( fd, "#!/bin/sh\n" );
  fprintf( fd, "command=\"$*; echo 'Press <return> to continue'\"\n" );
  fprintf( fd, "eval $command\n" );
  fprintf( fd, "read x\n" );
  fclose( fd );
  chmod( tmpFilename, 0750 );
  return tmpFilename;
}
